using System;
using System.Collections;
using UnityEngine;
using UnityEngine.UI;
using Firebase.Database;
using Unity.VisualScripting;

namespace Assets
{
    public class RealTime : MonoBehaviour
    {
        InputField _inputName;
        InputField _inputGold;
        Text _textName;
        Text _textGold;

        string _userID;
        DatabaseReference _databaseReference;
        
        void Awake()
        {
            _userID = SystemInfo.deviceUniqueIdentifier;
            
            _inputName = transform.Find("InputName").GetComponent<InputField>();
            _inputGold = transform.Find("InputGold").GetComponent<InputField>();
            _textName = transform.Find("TextName").GetComponent<Text>();
            _textGold = transform.Find("TextGold").GetComponent<Text>();
        }
        
        void Start() {
            // Get the root reference location of the database.
            _databaseReference = FirebaseDatabase.DefaultInstance.RootReference;
        }

        public void CreatUser()
        {
            User newUser = new User(_inputName.text, int.Parse(_inputGold.text));
            string json = JsonUtility.ToJson(newUser);
            _databaseReference.Child("users").Child("userID").SetRawJsonValueAsync(json);
        }

        IEnumerator GetName(Action<string> onCallBack)
        {
            var userNameDate = _databaseReference.Child("users").Child("userID").Child("UserName").GetValueAsync();

            yield return new WaitUntil(() => userNameDate.IsCompleted);

            if (userNameDate != null)
            {
                DataSnapshot snapshot = userNameDate.Result;

                onCallBack.Invoke(snapshot.Value.ToString());
            }
        }
        
        IEnumerator GetGold(Action<int> onCallBack)
        {
            var userGoldDate = _databaseReference.Child("users").Child("userID").Child("Gold").GetValueAsync();

            yield return new WaitUntil(() => userGoldDate.IsCompleted);

            if (userGoldDate != null)
            {
                DataSnapshot snapshot = userGoldDate.Result;

                onCallBack.Invoke(int.Parse(snapshot.Value.ToString()));
            }
        }

        public void GetUserInfo()
        {
            StartCoroutine(GetName(userName =>
            {
                _textName.text = userName;
            }));
            
            StartCoroutine(GetGold(gold =>
            {
                _textGold.text = gold.ToString();
            }));
        }

        public void UpdateData()
        {
            _databaseReference.Child("users").Child("userID").Child("UserName").SetValueAsync(_textName.text);
            _databaseReference.Child("users").Child("userID").Child("Gold").SetValueAsync(_textGold.text);
        }
    }

    public class User
    {
        public string UserName;
        public int Gold;

        public User(string userName, int gold)
        {
            this.UserName = userName;
            this.Gold = gold;
        }
    }
}
